@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009, 2010 Net-ng.
+# Copyright (c) 2008-2012 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -90,6 +90,9 @@ def absolute_url(url, static):
     Return:
       - an absolute URL
     """
+    if url.startswith('#'):
+        return url
+
     i = url.find(':')
     if ((i == -1) or not url[:i].isalpha()) and url and (url[0] != '/'):
         # If this is a relative URL, it's relative to the statics directory
@@ -352,7 +355,7 @@ class Form(xhtml_base._HTMLTag):
         Delete the existing ``<form>`` tags in the child tree
         """
         if isinstance(child, ET.ElementBase):
-            for f in child.findall('form'):
+            for f in child.findall('.//form'):
                 f.replace(f.getchildren()[1:])
 
         if isinstance(child, Form):
@@ -425,7 +428,7 @@ class TextInput(_HTMLActionTag):
         if hasattr(self, '_authorized_attribs'):
             element._authorized_attribs = self._authorized_attribs
 
-        return element(*children, **attrib)
+        return element(**dict(self.attrib))(*children, **attrib)
 
 
 class TextArea(_HTMLActionTag):
@@ -603,6 +606,20 @@ class ImageInput(_HTMLActionTag):
         """
         return self._async_action(renderer, action, with_request, permissions, subject)
 
+    def add_child(self, child):
+        """Add attributes to the image input
+
+        In:
+          - ``child`` -- attributes dictionary
+        """
+        super(ImageInput, self).add_child(child)
+
+        # If this is a relative URL, it's relative to the statics directory
+        # of the application
+        src = self.get('src')
+        if src is not None:
+            self.set('src', absolute_url(src, self.renderer.head.static_url))
+
 # ----------------------------------------------------------------------------------
 
 class Option(xhtml_base._HTMLTag):
@@ -623,7 +640,7 @@ class Option(xhtml_base._HTMLTag):
         if 'selected' in self.attrib:
             del self.attrib['selected']
 
-        if self.get('value') in values:
+        if self.get('value') in map(unicode, values):
             self.set('selected', 'selected')
 
         return self
@@ -845,6 +862,8 @@ def add_child(next_method, self, style):
 class Renderer(xhtml_base.Renderer):
     """The XHTML synchronous renderer
     """
+    XML_DOCTYPE = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
+    HTML_DOCTYPE = '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">'
 
     head_renderer_factory = HeadRenderer
 
@@ -852,7 +871,7 @@ class Renderer(xhtml_base.Renderer):
     # ---------------------------------------------
 
     a = TagProp('a', set(xhtml_base.allattrs+xhtml_base.focusattrs+('charset', 'type', 'name', 'href', 'hreflang', 'rel', 'rev', 'shape', 'coords', 'target', 'oncontextmenu')), A)
-    area =  area = TagProp('area', set(xhtml_base.allattrs + xhtml_base.focusattrs + ('shape', 'coords', 'href', 'nohref', 'alt', 'target')), A)
+    area = TagProp('area', set(xhtml_base.allattrs + xhtml_base.focusattrs + ('shape', 'coords', 'href', 'nohref', 'alt', 'target')), A)
     button = TagProp('button', set(xhtml_base.allattrs + xhtml_base.focusattrs + ('name', 'value', 'type', 'disabled')), SubmitInput)
     form = TagProp('form', set(xhtml_base.allattrs + ('action', 'method', 'name', 'enctype', 'onsubmit', 'onreset', 'accept_charset', 'target')), Form)
     img = TagProp('img', set(xhtml_base.allattrs+('src', 'alt', 'name', 'longdesc', 'width', 'height', 'usemap', 'ismap'
@@ -934,6 +953,32 @@ class Renderer(xhtml_base.Renderer):
         self.component = None
         self.model = None
 
+    def SyncRenderer(self, *args, **kw):
+        """Create an associated synchronous HTML renderer
+
+        Return:
+          - a new synchronous renderer
+        """
+        # If no arguments are given, this renderer becomes the parent of the
+        # newly created renderer
+        if not args and not kw:
+            args = (self,)
+
+        return self.__class__(*args, **kw)
+
+    def AsyncRenderer(self, *args, **kw):
+        """Create an associated asynchronous HTML renderer
+
+        Return:
+          - a new asynchronous renderer
+        """
+        # If no arguments are given, this renderer becomes the parent of the
+        # newly created renderer
+        if not args and not kw:
+            args = (self,)
+
+        return AsyncRenderer(*args, **kw)
+
     def makeelement(self, tag):
         """Make a tag
 
@@ -989,6 +1034,39 @@ class Renderer(xhtml_base.Renderer):
 
         return self._parse_html(parser, source, fragment, no_leading_text, **kw)
 
+    @property
+    def doctype(self):
+        """Generate the DOCTYPE of the document
+
+        If a doctype was set on the response object, use it
+        Else, use the HTML ou XHTML doctypes of this renderer
+
+        Return:
+          - the doctype
+        """
+        response = self.response
+
+        if response.doctype is not None:
+            return response.doctype
+
+        return self.XML_DOCTYPE if response.xml_output else self.HTML_DOCTYPE
+
+    @property
+    def content_type(self):
+        """Generate the content type of the document
+
+        If a content type was set on the response object, use it
+        Else, use the HTML ou XHTML content type of this renderer
+
+        Return:
+          - the content type
+        """
+        response = self.response
+
+        if response.content_type:
+            return response.content_type
+
+        return 'application/xhtml+xml' if response.xml_output else 'text/html'
 
     def start_rendering(self, component, model):
         """Method called before to render a component
@@ -1078,7 +1156,7 @@ class Renderer(xhtml_base.Renderer):
           - the completed url
         """
         i = u.find(':')
-        if ((i == -1) or not u[:i].isalpha()) and u and (u[0] != '/'):
+        if ((i == -1) or not u[:i].isalpha()) and (not u or (u[0] != '/')):
             u = self.url + '/' + u
 
         if params is None:
@@ -1174,17 +1252,54 @@ class AsyncRenderer(Renderer):
     """
     head_renderer_factory = AsyncHeadRenderer
 
-    def __init__(self, *args, **kw):
+    def __init__(self, parent=None, session=None, request=None, response=None, callbacks=None, static_url='', static_path='', url='/', async_header=False):
         """Renderer initialisation
 
         In:
           - ``parent`` -- parent renderer
+          - ``session`` -- the session object
+          - ``request`` -- the request object
+          - ``response`` -- the response object
+          - ``callbacks`` -- the registered actions on the tags
+          - ``static_url`` -- url of the static contents of the application
+          - ``static_path`` -- path of the static contents of the application
+          - ``url`` -- url prefix of the application
+          - ``async_header`` -- is the head renderer to create a synchronous or an asynchronous one?
         """
-        super(AsyncRenderer, self).__init__(*args, **kw)
+        super(AsyncRenderer, self).__init__(parent, session, request, response, callbacks, static_url, static_path, url)
+
+        if not (parent or async_header):
+            self.head = HeadRenderer(static_url=static_url)
 
         self.async_root = True;
         self.wrapper_to_generate = False    # Add a ``<div>`` around the rendering ?
 
+    def SyncRenderer(self, *args, **kw):
+        """Create an associated synchronous HTML renderer
+
+        Return:
+          - a new synchronous renderer
+        """
+        # If no arguments are given, this renderer becomes the parent of the
+        # newly created renderer
+        if not args and not kw:
+            args = (self,)
+
+        return Renderer(*args, **kw)
+
+    def AsyncRenderer(self, *args, **kw):
+        """Create an associated asynchronous HTML renderer
+
+        Return:
+          - a new asynchronous renderer
+        """
+        # If no arguments are given, this renderer becomes the parent of the
+        # newly created renderer
+        if not args and not kw:
+            args = (self,)
+
+        return self.__class__(*args, **kw)
+
     def javascript_url(self, url):
         self.head.javascript_url(url)
 
