@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009, 2010 Net-ng.
+# Copyright (c) 2008-2012 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -289,7 +289,7 @@ class Renderer(xml.XmlRenderer):
         source.close()
 
         body = ET.parse(html, parser).getroot()[0]
-        for e in body[:]:
+        for e in body:
             if isinstance(e, _HTMLTag):
                 # Attach the renderer to each roots
                 e._renderer = self
