@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009, 2010 Net-ng.
+# Copyright (c) 2008-2012 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -185,6 +185,7 @@ class _Tag(ET.ElementBase):
 
         for thing in iterable:
             clone = copy.deepcopy(element)
+            clone._renderer = element.renderer
             parent.append(clone)
 
             yield (clone, thing)
@@ -208,8 +209,8 @@ class _Tag(ET.ElementBase):
                 children = children[0]
             self.add_child(children)
 
-        if CHECK_ATTRIBUTES and not frozenset(self.attrib).issubset(self._authorized_attribs):
-            raise AttributeError("Bad attributes for element <%s>: " % self.tag) + ', '.join(list(frozenset(self.attrib) - self._authorized_attribs))
+        if CHECK_ATTRIBUTES and self._authorized_attribs and not frozenset(self.attrib).issubset(self._authorized_attribs):
+            raise AttributeError("Bad attributes for element <%s>: " % self.tag + ', '.join(list(frozenset(self.attrib) - self._authorized_attribs)))
 
         return self
 
@@ -407,7 +408,7 @@ class TagProp(object):
 
     Each time this attribute is read, a new tag is created
     """
-    def __init__(self, name, authorized_attribs, factory=None):
+    def __init__(self, name, authorized_attribs=None, factory=None):
         """Initialization
 
         In:
@@ -419,7 +420,7 @@ class TagProp(object):
         self._factory = factory
 
         if CHECK_ATTRIBUTES:
-            self._authorized_attribs = frozenset(authorized_attribs)
+            self._authorized_attribs = frozenset(authorized_attribs) if authorized_attribs is not None else None
 
     def __get__(self, renderer, cls):
         """Create a new tag each time this attribute is read
@@ -477,6 +478,9 @@ class XmlRenderer(common.Renderer):
     """
     __metaclass__ = RendererMetaClass
 
+    doctype = ''
+    content_type = 'text/xml'
+
     @classmethod
     def class_init(cls, special_tags):
         """Class initialisation
@@ -554,11 +558,15 @@ class XmlRenderer(common.Renderer):
         children = self._stack[0].getchildren()
 
         text = self._stack[0].text
+
         if text is not None:
             children.insert(0, text)
 
+        if not children:
+            return ''
+
         if len(children) == 1:
-            children = children[0]
+            return children[0]
 
         return children
 
