@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009, 2010 Net-ng.
+# Copyright (c) 2008-2012 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -17,11 +17,12 @@ expired session id, it creates a new root component and a new session.
 from __future__ import with_statement
 
 import sys
+import os
 
 import webob
 from webob import exc, acceptparse
 
-from nagare import component, presentation, serializer, database, top, security, log, comet
+from nagare import component, presentation, serializer, database, top, security, log, comet, i18n, local
 from nagare.security import dummy_manager
 from nagare.callbacks import Callbacks, CallbackLookupError
 from nagare.namespaces import xhtml
@@ -30,26 +31,32 @@ from nagare.sessions import ExpirationError
 
 # ---------------------------------------------------------------------------
 
-class MIMEAcceptWithoutWildcards(acceptparse.Accept):
-    def _match(self, item, match):
-        if '*' in item:
-            return False
-        return super(MIMEAcceptWithoutWildcards, self)._match(item, match)
+class Response(webob.Response):
+    def __init__(self, accept):
+        super(Response, self).__init__(headerlist=[])
+
+        accept = acceptparse.Accept(accept)
+        self.xml_output = accept.best_match(('text/html', 'application/xhtml+xml')) == 'application/xhtml+xml'
+
+        self.content_type = ''
+        self.doctype = None
 
 # ---------------------------------------------------------------------------
 
 class WSGIApp(object):
+    renderer_factory = xhtml.Renderer   # Default renderer
+
     def __init__(self, root_factory):
         """Initialization
 
         In:
           - ``root_factory`` -- function to create the application root component
-          - ``databases`` -- the SQLAlchemy metadata objects and the database engines settings
         """
         self.root_factory = root_factory
 
         self.static_path = ''
         self.static_url = ''
+        self.data_path = ''
         self.databases = []
         self.name = ''
         self.project_name = ''
@@ -60,6 +67,8 @@ class WSGIApp(object):
 
         self.security = dummy_manager.Manager()
 
+        self.set_default_locale(i18n.Locale())
+
     def set_config(self, config_filename, config, error):
         """Read the configuration parameters
 
@@ -89,6 +98,14 @@ class WSGIApp(object):
         """
         self.static_url = static_url
 
+    def set_data_path(self, data_path):
+        """Register the directory of the data
+
+        In:
+          - ``data_path`` -- the directory where the data of the application are located
+        """
+        self.data_path = data_path
+
     def set_publisher(self, publisher):
         """Register the publisher
 
@@ -113,12 +130,6 @@ class WSGIApp(object):
         """
         self.databases = databases
 
-    def start(self):
-        """Call after each process start
-        """
-        for database_settings in self.databases:
-            database.set_metadata(*database_settings)
-
     def set_project(self, name):
         """The application distribution name
 
@@ -127,6 +138,31 @@ class WSGIApp(object):
         """
         self.project_name = name
 
+    def set_default_locale(self, locale):
+        """Register the default locale
+
+        In:
+          - ``locale`` -- the default locale
+        """
+        self.default_locale = locale
+
+    def set_locale(self, locale):
+        """Set the locale of the request scope
+
+        In:
+          - ``locale`` -- the locale
+        """
+        if locale.dirname is None:
+            locale.dirname = os.path.join(self.data_path, 'locale')
+
+        i18n.set_locale(locale)
+
+    def start(self):
+        """Call after each process start
+        """
+        for database_settings in self.databases:
+            database.set_metadata(*database_settings)
+
     # -----------------------------------------------------------------------
 
     def on_bad_http_method(self, request, response):
@@ -240,15 +276,27 @@ class WSGIApp(object):
           - ``response`` -- the web response object
           - ``callbacks`` -- object to register the callbacks
         """
-        renderer_factory = xhtml.AsyncRenderer if async else xhtml.Renderer
-        return renderer_factory(
-                                None,
-                                session,
-                                request, response,
-                                callbacks,
-                                self.static_url, self.static_path,
-                                request.script_name
-                               )
+        renderer = self.renderer_factory(
+                                            None,
+                                            session,
+                                            request, response,
+                                            callbacks,
+                                            self.static_url, self.static_path,
+                                            request.script_name
+                                        )
+
+        if async:
+            renderer = renderer.AsyncRenderer(
+                                                None,
+                                                session,
+                                                request, response,
+                                                callbacks,
+                                                self.static_url, self.static_path,
+                                                request.script_name,
+                                                async_header = True
+                                             )
+
+        return renderer
 
     def start_request(self, root, request, response):
         """A new request is received, setup its dedicated environment
@@ -261,6 +309,8 @@ class WSGIApp(object):
         security.set_manager(self.security) # Set the security manager
         security.set_user(self.security.create_user(request, response)) # Create the User object
 
+        self.set_locale(self.default_locale) # Set the default Locale
+
     # Processing phase
     def _phase1(self, request, response, callbacks):
         """Phase 1 of the request processing:
@@ -279,7 +329,7 @@ class WSGIApp(object):
         return callbacks.process_response(request, response)
 
     # Rendering phase
-    def _phase2(self, request, response, output, is_xhr):
+    def _phase2(self, output, content_type, doctype, is_xhr, response):
         """Final step of the phase 2
 
         Phase 2 of the request processing:
@@ -288,15 +338,15 @@ class WSGIApp(object):
           - No modification of the objects is allowed
 
         In:
-          - ``request`` -- the web request object
-          - ``response`` -- the web response object
-          - ``output`` -- the rendered tree
+          - ``output`` -- the rendered content
+          - ``content_type`` -- the rendered content type
+          - ``doctype`` -- the (optional) doctype
           - ``is_xhr`` -- is the request a XHR request ?
 
-        Return:
-          - the content to send back to the browser
+        Out:
+          - ``response`` -- the response object
         """
-        response.body = serializer.serialize(output, request, response, not is_xhr)
+        (response.content_type, response.body) = serializer.serialize(output, content_type, doctype, not is_xhr)
         response.charset = 'utf-8'
 
     def __call__(self, environ, start_response):
@@ -309,19 +359,22 @@ class WSGIApp(object):
         Return:
           - the content to send back to the browser
         """
+        local.request.clear()
+
         # Create the ``WebOb`` request and response objects
         # -------------------------------------------------
 
         request = webob.Request(environ, charset='utf-8')
-        response = webob.Response(headerlist=[])
-
-        accept = MIMEAcceptWithoutWildcards('Accept', 'text/html' if self.always_html else str(request.accept))
-        response.xhtml_output = accept.best_match(('text/html', 'application/xhtml+xml')) == 'application/xhtml+xml'
+        response = Response('text/html' if self.always_html else str(request.accept))
 
         channel_id = request.params.get('_channel')
         nb = request.params.get('_nb')
         if channel_id and nb:
-            comet.channels.connect(environ['wsgi.multiprocess'], channel_id, int(nb), response)
+            if environ['wsgi.multiprocess']:
+                response.status = 501 # "Not Implemented"
+            else:
+                comet.channels.connect(channel_id, int(nb), environ['wsgi.input'].file.fileno(), response)
+
             return response(environ, start_response)
 
         xhr_request = request.is_xhr or ('_a' in request.params)
@@ -330,7 +383,7 @@ class WSGIApp(object):
         self.last_exception = None
 
         log.set_logger('nagare.application.'+self.name) # Set the dedicated application logger
-
+        
         # Create a database transaction for each request
         with database.session.begin():
             try:
@@ -381,7 +434,7 @@ class WSGIApp(object):
                 # Phase 2
                 # -------
 
-                # If the ``redirect_after_post`` parameter of the ``[application``
+                # If the ``redirect_after_post`` parameter of the ``[application]``
                 # section is `True`` (the default), conform to the PRG__ pattern
                 #
                 # __ http://en.wikipedia.org/wiki/Post/Redirect/GetPRG
@@ -403,11 +456,12 @@ class WSGIApp(object):
                             output = self.on_back(request, response, renderer, output)
 
                         if not xhr_request:
-                            output = top.wrap(response.content_type, renderer, output)
+                            output = top.wrap(renderer.content_type, renderer, output)
 
-                        self._phase2(request, response, output, render is not None)
+                        self._phase2(output, renderer.content_type, renderer.doctype, render is not None, response)
 
                         if not xhr_request:
+                            # TODO: move it into a ``renderer.end_rendering()`` method
                             callbacks.clear_not_used(renderer._rendered)
 
                     # Store the session
@@ -415,7 +469,7 @@ class WSGIApp(object):
 
                     security.get_manager().end_rendering(request, response, session)
                 except exc.HTTPException, response:
-                    # When a ``webob.exc`` object is raised during phase 2, stop immediatly
+                    # When a ``webob.exc`` object is raised during phase 2, stop immediately
                     # use it as the response object
                     pass
                 except Exception:
@@ -423,7 +477,7 @@ class WSGIApp(object):
                     response = self.on_exception(request, response)
             finally:
                 if session:
-                    session.lock.release()
+                    session.release()
 
         return response(environ, start_response)
 
